
public class Node {

    protected Object a;
    protected Node d;

    protected Node(Object ap, Node dp) {
	a = ap; d = dp; 
    }

    protected Object car() { return a; }
    protected Node cdr() { return d; }

    protected void setcar(Object ap) { a = ap; }
    protected void setcdr(Node dp) { d = dp; }


    public static Node cons(Object a, Node d) {
	return new Node(a,d);
    }

    public static Object car(Node n) {
	if ( n == null ) return null;
	return n.car();
    }

    public static Node cdr(Node n) {
	if ( n == null ) return null;
	return n.cdr();
    }

    public static void setcar(Node n, Object a) {
	if ( n == null ) return;
	n.setcar(a);
    }

    public static void setcdr(Node n, Node d) {
	if ( n == null ) return;
	n.setcdr(d);
    }

    public String toString() {
	String s = "(";
	Node t = this;
	Object a;
	for (;;) { 
            a = t.car();
	    if ( a == null ) s += "null";
	    else s += a.toString();
	    if ( t.cdr() != null ) {
               s += " ";
               t = t.cdr();
            } else break;
	}
	s += ")";
	return s;
    }


    protected Node append(Node dp) { 
	if ( d == null ) return cons( a, dp );
	Node t;
	if ( d.cdr() == null ) t = cons( d.car(), dp);
	else t = cons( d.car(), d.cdr().append( dp ) );
	return cons(a, t);
    }

    public static Node append(Node x, Node y) {
	if ( x == null ) return y;
	return x.append(y);
    }


    protected Object ncar(int j) { 
	Node t = this;
	for (int i = 0; i < j; i++ ) {
	    t = t.cdr();
            if ( t == null ) return null;
	}
	return t.car();
    }

    public static Object ncar(Node n, int j) {
        if ( n == null ) return null;
	return n.ncar(j);
    }

    protected int length() { 
	int l = 1;
	Node t = this;
	while ( t.cdr() != null ) {
	    t = t.cdr();
	    l++;
	}
	return l;
    }

    public static int length(Node n) {
        if ( n == null ) return 0;
	return n.length();
    }


    protected Node reverse() { 
	if ( d == null ) return this;
        return append( d.reverse(), cons( a, null) );
    }

    public static Node reverse(Node n) {
        if ( n == null ) return n;
	return n.reverse();
    }

    protected Object clone() { 
	Object ap = a;
	if ( a != null && a instanceof Node ) ap = ((Node)a).clone();
	if ( d == null ) return cons( ap, d );
	return cons( ap, (Node)d.clone() );
    }

    public static Object clone(Node n) {
        if ( n == null ) return n;
	return n.clone();
    }

    public boolean equals(Object np) { 
	if ( this == np ) return true;
	if ( np == null ) return false;
	if ( !(np instanceof Node) ) return false;
	Node n = (Node) np;
	if ( a == null ) {
           if ( n.car() != null ) return false;
	} else {
	    if ( a instanceof Node ) {
		if ( ! (n.car() instanceof Node) ) return false;
                if ( ! ((Node)a).equals( (Node)n.car() ) ) return false;
	    } else {
		if ( n.car() instanceof Node ) return false;
                if ( ! a.equals( n.car() ) ) return false;
	    }
	}
	if ( d == null ) return n.cdr() == null;
	return d.equals( n.cdr() );
    }

    public static boolean equals(Node a, Object b) {
        if ( a == null ) return b == null;
	return a.equals(b);
    }

}
