
/**
 * Klasse zum Rechnen mit Komplexen Zahlen
 * hk, 9.12.2001
 *
 **/

public class Complex {

    /* Die Datenstruktur */

    private double re;  // der Realteil
    private double im;  // der Imaginrteil


    /* Die Konstruktoren erzeugen aus einem Paar von 
       double Zahlen eine Komplexe Zahl */

    public Complex(double r, double i) {
        re = r;
        im = i;
    }

    public Complex(double r) {
        this(r,0.0);
    }

    public Complex() {
        this(0.0);
    }


    /* Konstanten: 1, 0 und i */

    public static final Complex ZERO = new Complex();

    public static final Complex ONE = new Complex(1.0);

    public static final Complex I = new Complex(0.0,1.0);


    /* Bestimmen des Real- und Imaginr-Teils */

    public double reTeil() { return re; }

    public double imTeil() { return im; }


    /* Darstellung als Zeichenkette */

    public String toString() {
        String s = "" + re;
        if ( im < 0 ) s += " - " + (-im) + "i";
           else s += " + " + im + "i";
        return s;
    }


    /* Vergleich zweier Komplexer Zahlen */

    public boolean equals(Complex b) {
	return ( re == b.reTeil() && im == b.imTeil() );
    }


    /* Arithmetische Operationen: +, -, - */

    public Complex add(Complex b) {
	return new Complex(re + b.reTeil(), im + b.imTeil());
    }

    public Complex subtract(Complex b) {
	return new Complex(re - b.reTeil(), im - b.imTeil());
    }

    public Complex negate() {
	return new Complex(-re,-im);
    }


    /* Arithmetische Operationen: Konjugierte, Absolut Betrag */

    public Complex conjugate() {
	return new Complex(re,-im);
    }

    public double abs() {
	return ( re * re + im * im );
    }


    /* Arithmetische Operationen: *, Inverse, / */

    public Complex multiply(Complex b) {
	return new Complex(re * b.reTeil() - im * b.imTeil(),
                           re * b.imTeil() + im * b.reTeil() );
    }

    public Complex inverse() {
        double a = this.abs();
	return new Complex(re/a, -im/a); // Konjugierte
    }

    public Complex divide (Complex b) {
	return this.multiply( b.inverse() );
    }

    

}
