
import java.io.IOException;
import java.io.PrintWriter;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.AttributesImpl;
import org.w3c.dom.Document; 


public class SAXRechnung extends SAXSchreiber {

    DOMLager lager = null;

    public SAXRechnung(PrintWriter p, DOMLager d) {
	pw = p;
        lager = d;
    }

    boolean getAnzahl = false;
    long anzahl = 0;
    String aNummer = "";
    String einheit = "EUR";
    double summe = 0.0;
    AttributesImpl einheitAttribut = null;

    public void startDocument() {
        super.startDocument();
        pw.println("<!DOCTYPE rechnung SYSTEM \"rechnung.dtd\" >");
        pw.flush();
        einheitAttribut = new AttributesImpl();
	einheitAttribut.addAttribute(null,"einheit",null,
                                     "CDATA",einheit);
    }


    public void startElement(String nameuri, String local, String quali, 
                             Attributes attrs) {
        if ( local.equals("lieferung") ) {
           super.startElement(nameuri,"rechnung",null,attrs);
           pw.flush();
           return;
	}
        if ( local.equals("artikel") ) {
           super.startElement(nameuri,local,quali,attrs);
           String aNr = attrs.getValue("nr");
           aNummer = aNr;
           return;
	}
        if ( local.equals("anzahl") ) {
	    getAnzahl = true;
	}
        super.startElement(nameuri,local,quali,attrs);
    } 


    public void endElement(String nameuri, String local, String quali) {
        if ( local.equals("lieferung") ) {
           super.startElement(nameuri,"summe",null,einheitAttribut);
	   String s = "" + summe;
           char[] bs = s.toCharArray();
	   super.characters(bs,0,bs.length);
           super.endElement(nameuri,"summe",null);
           pw.flush();
           s = "\n";
           bs = s.toCharArray();
	   super.characters(bs,0,bs.length);

           super.endElement(nameuri,"rechnung",null);
           pw.flush();
           return;
	}
        if ( local.equals("artikel") ) {
           super.startElement(nameuri,"epreis",null,einheitAttribut);
           String aNr = aNummer;
           String eh = lager.getPreisEinheitNummer(aNr);
           if ( !eh.equals(einheit) ) {
	       System.err.println("Einheiten Umrechnung notwendig von " 
                 + eh + " nach " + einheit);
	   }
	   String preis = lager.getPreisNummer(aNr);
	   String s = "" + preis;
           char[] bs = s.toCharArray();
	   super.characters(bs,0,bs.length);
           super.endElement(nameuri,"epreis",null);
           pw.flush();
           s = "\n";
           bs = s.toCharArray();
	   super.characters(bs,0,bs.length);

           super.startElement(nameuri,"gpreis",null,einheitAttribut);
	   double gpreis = Double.parseDouble(preis);
	   gpreis *= anzahl;
           summe += gpreis;
	   s = "" + gpreis;
           bs = s.toCharArray();
	   super.characters(bs,0,bs.length);
           super.endElement(nameuri,"gpreis",null);
           pw.flush();
           s = "\n";
           bs = s.toCharArray();
	   super.characters(bs,0,bs.length);
	}
        super.endElement(nameuri,local,quali);
    } 


    public void characters(char ch[], int start, int length) {
        super.characters(ch,start,length);
        if ( !getAnzahl ) {
	   return;
	}
        getAnzahl = false;
        String s = new String(ch,start,length);
        anzahl = Long.parseLong(s);
    } 

}
